#
# anm_rescale version 3
#
import Blender
import os,math,struct,pdb
import struct
def pad(stream):
    align = stream.tell()
    if ((align % 4)>0): 
        np = 4 - (align % 4)
        for i in range(np):
            b = stream.read(1)    
def pad_output(stream):
    align = stream.tell()
    if ((align % 4)>0): 
        np = 4 - (align % 4)
        for i in range(np):
            b = stream.write(chr(0))
def pad_byte_string(byte_string):
    align = len(byte_string)
    pd = ""
    if((align % 4)>0):
        np = 4 - (align % 4)
        for i in range(np):
            pd += chr(0)
    return pd
            
class AnmHeader:
    def __init__(self,stream):
        (self.magic,) = struct.unpack('L',stream.read(4))
        (self.count_anims,self.count_static) = struct.unpack('BB',stream.read(2))
        (self.frame_count,) = struct.unpack('H',stream.read(2))
        (self.m11,self.m12,self.m13) = struct.unpack('fff',stream.read(12))
        (self.m21,self.m22,self.m23) = struct.unpack('fff',stream.read(12))
        (self.m31,self.m32,self.m33) = struct.unpack('fff',stream.read(12))
        (self.x,self.y,self.z) = struct.unpack('fff',stream.read(12))
        (self.flags,) = struct.unpack('B',stream.read(1))
        pad(stream)
    def details(self):
        print 'AnmHeader:'
        print 'magic = ',self.magic
        print 'count_anims = ',self.count_anims
        print 'count_static = ',self.count_static
        print 'frame_count = ',self.frame_count
        print 'world origin = '
        print '%10.6f %10.6f %10.6f'%(self.m11,self.m12,self.m13)
        print '%10.6f %10.6f %10.6f'%(self.m21,self.m22,self.m23)
        print '%10.6f %10.6f %10.6f'%(self.m31,self.m32,self.m33)
        print '%10.4f %10.4f %10.4f'%(self.x,self.y,self.z)
    def save(self,stream):
        bytestring = struct.pack('LBBH',self.magic,self.count_anims,self.count_static,self.frame_count)
        bytestring += struct.pack('fff',self.m11,self.m12,self.m13)
        bytestring += struct.pack('fff',self.m21,self.m22,self.m23)
        bytestring += struct.pack('fff',self.m31,self.m32,self.m33)
        bytestring += struct.pack('fff',self.x,self.y,self.z)
        bytestring += struct.pack('Bxxx',self.flags)
        stream.write(bytestring)
        pad_output(stream)
class Origin:
    def __init__(self,stream):
        (self.m11,self.m12,self.m13) = struct.unpack('fff',stream.read(12))
        (self.m21,self.m22,self.m23) = struct.unpack('fff',stream.read(12))
        (self.m31,self.m32,self.m33) = struct.unpack('fff',stream.read(12))
        (self.x,self.y,self.z) = struct.unpack('fff',stream.read(12))
    def save(self,stream):
        bytestring = struct.pack('fff',self.m11,self.m12,self.m13)
        bytestring += struct.pack('fff',self.m21,self.m22,self.m23)
        bytestring += struct.pack('fff',self.m31,self.m32,self.m33)
        bytestring += struct.pack('fff',self.x,self.y,self.z)
        stream.write(bytestring)
    def details(self):
        print 'origin = '
        print '%10.6f %10.6f %10.6f'%(self.m11,self.m12,self.m13)
        print '%10.6f %10.6f %10.6f'%(self.m21,self.m22,self.m23)
        print '%10.6f %10.6f %10.6f'%(self.m31,self.m32,self.m33)
        print '%10.4f %10.4f %10.4f'%(self.x,self.y,self.z)
class BoneAnmHeader:
    def __init__(self,stream,scale):
        
        (fc,bc,besson,flags) = struct.unpack('LHBB',stream.read(8))
        (x,y,z,a) = struct.unpack('ffff',stream.read(16))
        (prob,compression) = struct.unpack('ff',stream.read(8))
        boneparentlist = ()
        for bn in range(bc):
            boneparentlist += struct.unpack('B',stream.read(1))
        self.frame_count = fc
        self.bone_count = bc
        self.besson_type = besson
        self.flags = flags
        self.dest_x = x*scale
        self.dest_y = y*scale
        self.dest_z = z*scale
        self.dest_angle = a
        self.probability = prob
        self.compression = compression*scale
        self.bonelist = boneparentlist
        self.scale = scale
        pad(stream)
    def save(self,stream):
        bytestring = struct.pack('LHBBffffff',
                                 self.frame_count,
                                 self.bone_count,
                                 self.besson_type,
                                 self.flags,
                                 self.dest_x,
                                 self.dest_y,
                                 self.dest_z,
                                 self.dest_angle,
                                 self.probability,
                                 self.compression)
        for nb in range(self.bone_count):
            bytestring += struct.pack('B',self.bonelist[nb])
        stream.write(bytestring)
        pad_output(stream)
    def details(self):
        print ''
        print 'BoneAnmHeader:'
        print 'frame_count = ',self.frame_count
        print 'bone_count  = ',self.bone_count
        print 'besson_type = ',self.besson_type
        print 'flags       = ',self.flags
        print 'dest_x      = ',self.dest_x
        print 'dest_y      = ',self.dest_y
        print 'dest_z      = ',self.dest_z
        print 'dest_angle  = ',self.dest_angle
        print 'probability = ',self.probability
        print 'compression = ',self.compression
        print 'bonelist    = ',self.bonelist
    def pack(self):
        bytestring = struct.pack('LHBB',self.framecount,
                                 self.bonecount,
                                 self.TypeForBesson,
                                 self.Flags)
        bytestring += struct.pack('ffff',self.dest_x,
                                  self.dest_y,
                                  self.dest_z,
                                  self.dest_angle)
        bytestring += struct.pack('f',self.compression)
        for i in range(self.bonelist):
            bytestring += struct.pack('B',self.bonelist[i])
        byte_list = []
        for i in range(len(byte_list)):
            byte_list += byestring[i]
        return byte_list
def bone_frames(hdr,stream,ostream):
        for frame in range(hdr.frame_count):
            (mask,) = struct.unpack('H',stream.read(2))
            fc = hdr.frame_count
            byte_string += chr[0]
            byte_string += chr[0]
            while fc > 0:
                holdcount = 1
                (data,) = struct.unpack('H',stream.read(2))
                if (data & mask):
                    v1 = data & (mask - 1)
                    v2 = data - mask
                    v3 = ~(mask-1)
                    holdcount = v1 | ((v2 & v3) >> 1)
                    print 'holdcount = ',holdcount
                    if(holdcount>framecount) or (holdcount < 0):
                        raise ValueError, ' value outside bounds for hold.'
                    (holdvalue,) = struct.unpack('H',stream.read(2))
                    for hold in range(holdcount):
                        byte_string += struct.pack('H',holdvalue)    
class BoneAnm:
    def __init__(self,stream,hdr):
        trans_static_str = stream.read(4)
        (self.trans_static,pad1,pad2,pad3) = struct.unpack('BBBB',trans_static_str)
        self.byte_string = trans_static_str
        load_bone_frame_data(self,stream,hdr)
        load_bone_frame_data(self,stream,hdr)
        load_bone_frame_data(self,stream,hdr)
        self.byte_string += pad_byte_string(self.byte_string)
        pad(stream)
        if self.trans_static == 0:
            print 'loading u data'
            load_bone_frame_data(self,stream,hdr)
            print 'loading v data'
            load_bone_frame_data(self,stream,hdr)
            print 'loading w data'
            load_bone_frame_data(self,stream,hdr)
        else:
            print 'loading transtatic data'
            (x,y,z) = struct.unpack('fff',stream.read(12))
            scale = hdr.scale
            self.byte_string += struct.pack('fff',scale*x,scale*y,scale*z)
        pad(stream)
        self.byte_string += pad_byte_string(self.byte_string)
    def save(self,ostream):
        ostream.write(self.byte_string)
        
class prop_anm_header:
    def __init__(self,stream):
        self.byte_string = stream.read(4)
        (val,) = struct.unpack('Hxx',self.byte_string)
        has_scale = (val & 2)>>1
        if has_scale:
            self.byte_string += stream.read(4)
class PropAnim:
    def __init__(self, stream,frame_count):
        self.name = stream.read(32)
        self.header = prop_anm_header(stream)
        self.frames = stream.read(12*frame_count)
    def save(self,ostream):
        ostream.write(self.name)
        ostream.write(self.header.byte_string)
        ostream.write(self.frames)
        
        
def load_bone_frame_data(self,stream,hdr):
#    print 'file address = ',stream.tell()
#    print 'current byte_string = ',len(self.byte_string)
    sa = stream.tell()
    mask_str = stream.read(2)
    # force mask to zero
    self.byte_string += chr(0)
    self.byte_string += chr(0)
    (mask,) = struct.unpack('H',mask_str)

    print 'at sa = ',sa,'mask = ',mask
    fc = hdr.frame_count
    while fc > 0:
#        print 'fc = ',fc,' file address = ',stream.tell()
        holdcount = 1
        data_str = stream.read(2)
        (data,) = struct.unpack('H',data_str)
        if (data & mask):
            v1 = data & (mask - 1)
            v2 = data - mask
            v3 = ~(mask - 1)
            print 'file address: ',stream.tell()
            print 'data = ',data
            print 'mask = ',mask
            print 'v1,v2,v3 = ',v1,v2,v3
            holdcount = v1 | ((v2 & v3) >> 1)
            print 'holdcount = ',holdcount
            if (holdcount>hdr.frame_count) or (holdcount < 0):
                print 'Hold error'
                byte_list =[]
                for i in range(len(self.byte_string)):
                    byte_list += [ord(self.byte_string[i])]
#                print 'byte_list = ',byte_list
                raise ValueError, 'value is outside bounds for hold.'
            hold_str = stream.read(2)
            for hold in range(holdcount):
                self.byte_string += hold_str
        else:
            self.byte_string += data_str
        fc -= holdcount
def main(curdir):
    scn = Blender.Scene.GetCurrent()
    filename = scn.properties['ANMHead']['Target File']
    stream = open(filename,'rb')
    filepath = os.path.dirname(filename)
    filebase = os.path.basename(filename)
    fname = curdir + '\\r_'+filebase
    ostream = file(fname,'wb')
    anm_header = AnmHeader(stream)
    anm_header.save(ostream)

    for origins in range(anm_header.count_anims-1):
        origin = Origin(stream)
        origin.save(ostream)    
    for ba in range(anm_header.count_anims):
        scale = 1.0
        name = 'BoneAnmHdr'+str(ba+1)
        if scn.properties.has_key(name):
            if scn.properties[name].has_key('scale'):
                scale = scn.properties[name]['scale']
            else :
                scn.properties[name]['scale'] = 1.0
        else :
            scn.properties[name] = {}
            scn.properties[name]['scale'] = 1.0
        ba_header = BoneAnmHeader(stream,scale)
        ba_header.save(ostream)
#for bone in range(ba_header.bone_count):   
        for bone in range(ba_header.bone_count):
            print 'processing bone # ',bone
            bone_anm = BoneAnm(stream,ba_header)
            bone_anm.save(ostream)
    for pa in range(anm_header.count_static):
        prop_anim = PropAnim(stream,anm_header.frame_count)
        prop_anim.save(ostream)
#Blender.Window.FileSelector(main,"Rescale ANM",'*.anm')




